﻿Imports System.Drawing.Printing

Public Class MainForm
    ''' <summary>
    ''' Auflistung mit der Breite der einzelnen Spalten
    ''' </summary>
    Private ReadOnly SpaltenBreiten As New List(Of Single)
    ''' <summary>
    ''' Index der aktuell ausgegebenen DGV-Zeile
    ''' </summary>
    Private DGVZeilenIndex As Integer
    ''' <summary>
    ''' Breite der Tabelle
    ''' </summary>
    Private DGVDruckbreite As Single
    ''' <summary>
    ''' Rahmen um die Zelle
    ''' </summary>
    Private ReadOnly ZellPadding As Integer = 5

    ' Hintergrundfarben für Header und Zellen
    Private ReadOnly HeaderColor As Color = Color.LightGray
    Private ReadOnly RowColor As Color = Color.White
    Private ReadOnly RowColorAlternate As Color = Color.LightYellow

    ' Flags zur Druckausgabe => CheckBoxen
    Private CenterHorz As Boolean = True
    Private PrintGrating As Boolean = True
    Private ColumnsAutoSize As Boolean = False
    Private AlternateColor As Boolean = True

    ''' <summary>
    ''' Befüllen des DGV
    ''' </summary>
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        For i As Integer = 1 To 80
            Me.dgvTabelle.Rows.Add("Peter", "Punkt-Maier", i, i * i)
        Next

        Me.dgvTabelle.AllowUserToAddRows = False          ' Für Testzwecke

        Me.PrintDocument1.DefaultPageSettings.Landscape = False
    End Sub

    ''' <summary>
    ''' Sofortiger Druck der Tabelle
    ''' </summary>
    Private Sub Btn_Drucken_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDrucken.Click
        Me.SpaltenBreitenErmitteln()
        Me.PrintDocument1.Print()
    End Sub

    ''' <summary>
    ''' Anzeige der Druckvorschau
    ''' </summary>
    Private Sub Btn_Vorschau_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnVorschau.Click
        Me.SpaltenBreitenErmitteln()

        Using dlg As New PrintPreviewDialog
            dlg.Document = Me.PrintDocument1
            dlg.WindowState = FormWindowState.Maximized
            ' 2 Seiten anzeigen
            dlg.PrintPreviewControl.Columns = 2
            dlg.ShowDialog()
        End Using
    End Sub

    ''' <summary>
    ''' Ermitteln der Spaltenbreiten
    ''' </summary>
    Private Sub SpaltenBreitenErmitteln()
        Dim ZellInhalt As String
        Dim g As Graphics = dgvTabelle.CreateGraphics

        Me.SpaltenBreiten.Clear()
        Me.DGVDruckbreite = 0

        If ColumnsAutoSize Then
            For DGVSpalte = 0 To Me.dgvTabelle.Columns.Count - 1

                ' *** Breite das Textes im Spaltenkopf als Mindestbreite
                ZellInhalt = dgvTabelle.Columns(DGVSpalte).HeaderText
                Dim Breite = g.MeasureString(ZellInhalt, dgvTabelle.Font).Width + ZellPadding
                SpaltenBreiten.Add(Breite)

                ' *** Jede Spalte komplett durchlaufen und größte Textbreite finden und speichern
                For DGVZeile As Integer = 0 To dgvTabelle.Rows.Count - 1
                    ' Zellinhalt holen und ggf. formatieren
                    ZellInhalt = dgvTabelle.Rows(DGVZeile).Cells(DGVSpalte).Value.ToString
                    If Not String.IsNullOrEmpty(dgvTabelle.Columns(DGVSpalte).DefaultCellStyle.Format) Then
                        ZellInhalt = Convert.ToDouble(ZellInhalt).ToString(dgvTabelle.Columns(DGVSpalte).DefaultCellStyle.Format)
                    End If
                    ' Spaltenbreite feststellen und prüfen, ob größer als bisher und ggf. austauschen
                    Breite = g.MeasureString(ZellInhalt, dgvTabelle.Font).Width + ZellPadding
                    If SpaltenBreiten(DGVSpalte) < Breite Then
                        SpaltenBreiten(DGVSpalte) = Breite
                    End If
                Next
                DGVDruckbreite += SpaltenBreiten(DGVSpalte)
            Next
        Else
            ' Kein Spalten-AutoSizing
            For DGVSpalte As Integer = 0 To dgvTabelle.Columns.Count - 1
                Dim Breite = dgvTabelle.Columns(DGVSpalte).Width + ZellPadding
                SpaltenBreiten.Add(Breite)
                DGVDruckbreite += Breite
            Next
        End If
    End Sub

    ''' <summary>
    ''' Vorbereiten des Ausdrucks,
    ''' wird pro Druckvorgang genau einmalk aufgerufen
    ''' </summary>
    Private Sub PrintDocument1_BeginPrint(sender As Object, e As PrintEventArgs) Handles PrintDocument1.BeginPrint
        ' Ausgangssituation
        DGVZeilenIndex = 0
    End Sub

    ''' <summary>
    ''' der eigentliche Druck,
    ''' wird für jede einzelne Seite aufgerufen
    ''' </summary>
    Private Sub PrintDocument1_PrintPage(ByVal sender As Object, ByVal e As Printing.PrintPageEventArgs) Handles PrintDocument1.PrintPage
        Dim cellBox As New RectangleF
        Dim sf As New StringFormat

        Dim borderLeft As Single
        Dim borderRight As Single
        Dim borderTop As Single
        Dim borderBottom As Single

        Dim PageWidth As Integer
        Dim PageHeight As Decimal

        Dim dgvMaxRowIndex As Integer
        Dim dgvMaxColIndex As Integer

        Dim content As String
        Dim PosX As Integer
        Dim PosY As Integer
        Dim PosX1 As Integer

        e.Graphics.SmoothingMode = Drawing2D.SmoothingMode.AntiAlias
        e.Graphics.TextRenderingHint = Drawing.Text.TextRenderingHint.AntiAlias

        ' Höhe der Schrift ermitteln anhand der Schriftart(Font)
        ' mittels Mustertext 
        ' und Höhe geringfügig erhöhen

        Dim g As Graphics = dgvTabelle.CreateGraphics
        ' Ermittlung der Schrifthöhe (Text mit Ober- und Unterlängen)
        ' Font.Height ist nicht so hoch!
        Dim TextHöhe = g.MeasureString("Bg", dgvTabelle.Font).Height + ZellPadding

        With PrintDocument1.DefaultPageSettings
            borderLeft = .Margins.Left                          ' Rand links
            borderRight = .Margins.Right                        ' Rand rechts
            borderTop = .Margins.Top                            ' Rand oben
            borderBottom = .Margins.Bottom                      ' Rand unten

            PageWidth = .Bounds.Width - CInt(borderRight + borderLeft)      ' Druckbereich Breite
            PageHeight = .Bounds.Height - CInt(borderTop + borderBottom)    ' Druckbereich Höhe

            e.Graphics.DrawRectangle(Pens.Black, borderLeft, borderTop, PageWidth, PageHeight) ' Rahmen um alles
        End With

        If CenterHorz Then              ' Tabelle horizontal zentrieren
            borderLeft = CSng(borderLeft + (PageWidth / 2)) - (DGVDruckbreite / 2)
        End If

        ' Den höchsten ZeilenIndex des DataGridview ermitteln
        ' in Abhängigkeit davon, ob eine leere Zeile am Ende steht
        If dgvTabelle.AllowUserToAddRows Then
            dgvMaxRowIndex = dgvTabelle.Rows.Count - 2
        Else
            dgvMaxRowIndex = dgvTabelle.Rows.Count - 1
        End If

        dgvMaxColIndex = dgvTabelle.Columns.Count - 1

        sf.Alignment = StringAlignment.Near
        sf.LineAlignment = StringAlignment.Center

        PosY = CInt(borderTop)
        PosX = CInt(borderLeft)

        ' Spaltenköpfe drucken
        For DruckSpalte = 0 To dgvMaxColIndex
            cellBox = New RectangleF(PosX, PosY, SpaltenBreiten(DruckSpalte), TextHöhe)
            content = dgvTabelle.Columns(DruckSpalte).HeaderText

            e.Graphics.FillRectangle(New SolidBrush(HeaderColor), cellBox)
            If PrintGrating = True Then
                e.Graphics.DrawRectangle(Pens.Black, cellBox.X, cellBox.Y, cellBox.Width, cellBox.Height)
            End If
            e.Graphics.DrawString(content, dgvTabelle.Font, Brushes.Black, cellBox, sf)

            PosX += CInt(SpaltenBreiten(DruckSpalte))
        Next

        ' die aktuelle Ausgabeposition
        PosX = CInt(borderLeft)
        PosY = CInt(borderTop + TextHöhe)
        ' Zeilennummer auf der aktuellen Seite
        Dim DruckZeile = 0
        ' Schleife über alle Zeilen der aktuellen Seite
        Do
            ' Schleife über alle Spalten
            For DruckSpalte = 0 To dgvMaxColIndex
                Dim value = dgvTabelle.Rows(DruckZeile + DGVZeilenIndex).Cells(DruckSpalte).Value
                content = If(value Is Nothing, "", value.ToString)
                cellBox = New RectangleF(PosX, PosY, SpaltenBreiten(DruckSpalte), TextHöhe)

                If Not String.IsNullOrEmpty(dgvTabelle.Columns(DruckSpalte).DefaultCellStyle.Format.ToString) Then
                    content = Convert.ToDouble(content).ToString(dgvTabelle.Columns(DruckSpalte).DefaultCellStyle.Format.ToString)
                End If

                If AlternateColor Then
                    ' Zeilen abwechselnd einfärben
                    If ((DGVZeilenIndex + DruckZeile) Mod 2 = 0) Then
                        e.Graphics.FillRectangle(New SolidBrush(RowColor), cellBox)
                    Else
                        e.Graphics.FillRectangle(New SolidBrush(RowColorAlternate), cellBox)
                    End If
                End If

                If PrintGrating = True Then
                    e.Graphics.DrawRectangle(Pens.Black, cellBox.X, cellBox.Y, cellBox.Width, cellBox.Height)
                End If

                Select Case dgvTabelle.Columns(DruckSpalte).DefaultCellStyle.Alignment
                    Case DataGridViewContentAlignment.NotSet, DataGridViewContentAlignment.MiddleLeft
                        cellBox = New RectangleF(PosX, PosY, SpaltenBreiten(DruckSpalte), TextHöhe)

                    Case DataGridViewContentAlignment.MiddleRight
                        PosX1 = CInt(PosX + SpaltenBreiten(DruckSpalte) - (g.MeasureString(content, dgvTabelle.Font).Width))
                        cellBox = New RectangleF(PosX1, PosY, SpaltenBreiten(DruckSpalte), TextHöhe)

                    Case DataGridViewContentAlignment.MiddleCenter
                        PosX1 = CInt(PosX + (SpaltenBreiten(DruckSpalte) / 2) - (g.MeasureString(content, dgvTabelle.Font).Width / 2))
                        cellBox = New RectangleF(PosX1, PosY, SpaltenBreiten(DruckSpalte), TextHöhe)

                    Case Else
                        cellBox = New RectangleF(PosX, PosY, SpaltenBreiten(DruckSpalte), TextHöhe)

                End Select
                e.Graphics.DrawString(content, dgvTabelle.Font, Brushes.Black, cellBox, sf)

                PosX += CInt(SpaltenBreiten(DruckSpalte))
            Next

            PosX = CInt(borderLeft)
            PosY += CInt(TextHöhe)
            DruckZeile += 1
        Loop Until ((DruckZeile + DGVZeilenIndex) > (dgvMaxRowIndex)) OrElse ((PageHeight + borderTop) < PosY + TextHöhe)

        If (DruckZeile + DGVZeilenIndex) > dgvMaxRowIndex Then
            ' Tabelle fertig gedruckt
            e.HasMorePages = False
        Else
            ' es sind weitere Zeilen zu drucken
            e.HasMorePages = True
            DGVZeilenIndex = DruckZeile + DGVZeilenIndex
        End If
    End Sub
End Class